##
## screen.py -- Screen module, curses interface.
##
## Copyleft 1997 Teemu Kalvas <chery@s2.org>
##

## +method clean        ()
## +class Screen        ()
##  -method new_window  (name,
##                       server)
##  -method next        ()
##  -method prev        ()
##  -method touched
##  -method log_area
##  -method stat_area
##  -method edit_area
##  -method current
##  -method stat_line
##  -method edit_field

import curses
import signal
from types import *

import config
import editfield
import statline
import window

def clean():
    curses.nl()
    curses.noraw()
    curses.endwin()

class TextSpan:
    def __init__(self, str):
        if type(str) == ListType:
            self.spans = str
            return
        self.spans = []
        attr = 0
        i = 0
        j = 0
        for ch in str:
            if ch == '\002':
                if j > i:
                    self.spans.append((str[i:j], attr))
                attr = attr ^ curses.A_BOLD
                i = j + 1
            elif ch == '\037':
                if j > i:
                    self.spans.append((str[i:j], attr))
                attr = attr ^ curses.A_UNDERLINE
                i = j + 1
            elif ch == '\026':
                if j > i:
                    self.spans.append((str[i:j], attr))
                attr = attr ^ curses.A_REVERSE
                i = j + 1
            elif (ord(ch) & 127) < 32:
                if j > i:
                    self.spans.append((str[i:j], attr))
                i = j + 1
            j = j + 1
        if j > i:
            self.spans.append((str[i:j], attr))

    def __len__(self):
        return reduce(lambda a, x: a + len(x[0]), self.spans, 0)

    def __getitem__(self, i):
        for sp in self.spans:
            l = len(sp[0])
            if i < l:
                return sp[0][i]
            i = i - l

    def __getslice__(self, i, j):
        rspans = []
        for sp in self.spans:
            ll = len(sp[0])
            if i < ll:
                if j <= 0:
                    return TextSpan(rspans)
                rspans.append((sp[0][i:j], sp[1]))
            i = max(0, i - ll)
            j = max(0, j - ll)
        return TextSpan(rspans)

    def __add__(self, str):
        spans = self.spans[:]
        if spans[-1][1]:
            spans.append((str, 0))
        else:
            spans[-1] = spans[-1][0] + str, 0
        return TextSpan(spans)

    def print_line(self, window, y):
        x = 0
        k = 0
        l = len(self.spans)
        while k < l:
            str, attr = self.spans[k]
            ll = len(str)
            if k < l - 1:
                window.addstr(y, x, str, attr)
                x = x + ll
            elif ll > 0:
                window.addstr(y, x, str[:-1], attr)
                window.insch(y, x + ll - 1, ord(str[-1]), attr)
            k = k + 1

class Empty:
    name = '(none)'

    def __init__(self):
	self.next = self

class Screen:
    def __init__(self):
	self.window = curses.initscr()
	curses.raw()
	curses.noecho()
	curses.nonl()
	curses.intrflush(1)
	self.window.keypad(1)
	self.touched = 0
	height, width = self.window.getmaxyx()
	self.log_area = self.window.subwin(height - 2, width, 0, 0)
	self.log_area.scrollok(1)
	self.stat_area = self.window.subwin(1, width, height - 2, 0)
	self.stat_area.scrollok(1)
	self.edit_area = self.window.subwin(1, width, height - 1, 0)
	self.edit_area.scrollok(1)
	self.current = Empty()
	self.stat_line = statline.StatLine(self)
	self.edit_field = editfield.EditField(self)
	signal.signal(signal.SIGWINCH, self.resize)
        self.marked = None

    def resize(self, a, b):
	self.touched = 64

    def new_window(self, name, server):
	if self.current.__class__ <> Empty:
	    prev = self.current
	    next = prev.next
	    self.current = window.Window(self, next, prev, name, server)
	    prev.next = self.current
	    next.prev = self.current
	else:
	    self.current = window.Window(self, None, None, name, server)
	    self.current.next = self.current
	    self.current.prev = self.current
	self.touched = self.touched | 63
	return self.current

    def next(self):
	if self.current:
	    self.current = self.current.next
	    self.current.detect = 0
	    self.touched = self.touched | 63

    def prev(self):
	if self.current:
	    self.current = self.current.prev
	    self.current.detect = 0
	    self.touched = self.touched | 63

    def mark(self):
        if self.current:
            self.marked = self.current
            self.touched = self.touched | 63

    def to_marked(self):
        if self.marked:
            self.current = self.marked
            self.marked = None
            self.current.detect = 0
            self.touched = self.touched | 63

    def to_active(self):
        win = self.current.next
        while win <> self.current:
            if win.detect:
                self.current = win
                self.current.detect = 0
                self.touched = self.touched | 63
                break
            win = win.next

## End, ##
